--@ c:\workspace\golf\src\main\sql\create_tables.sql;

DROP TABLE SCORE;
DROP TABLE HOLE;
DROP TABLE ROUND;
DROP TABLE TEE;
DROP TABLE GOLFER;
DROP TABLE COURSE;

DROP SEQUENCE HOLE_SEQ;
DROP SEQUENCE ROUND_SEQ;
DROP SEQUENCE TEE_SEQ;
DROP SEQUENCE GOLFER_SEQ;
DROP SEQUENCE COURSE_SEQ;

CREATE SEQUENCE HOLE_SEQ START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE ROUND_SEQ START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE TEE_SEQ START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE GOLFER_SEQ START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE COURSE_SEQ START WITH 1 INCREMENT BY 1;

CREATE TABLE GOLFER(
	ID INTEGER PRIMARY KEY,
	USERNAME VARCHAR2(20) NOT NULL,
	PASSWORD VARCHAR2(20) NOT NULL,
	FIRST_NAME VARCHAR2(50),
	LAST_NAME VARCHAR2(50),
	BIRTHDATE DATE,
	EMAIL VARCHAR2(100),
	HANDICAP NUMBER
)
/

CREATE TABLE COURSE(
	ID INTEGER PRIMARY KEY,
  	NAME VARCHAR2(100)
)
/

CREATE TABLE TEE(
	ID INTEGER PRIMARY KEY,
	COURSE_ID INTEGER REFERENCES COURSE,
	TEE_TYPE VARCHAR2(20),
  	RATING NUMBER,
  	SLOPE INTEGER 
)
/

CREATE TABLE HOLE(
	ID INTEGER PRIMARY KEY,
	TEE_ID INTEGER REFERENCES TEE,
	HOLE_NO INTEGER,
	PAR INTEGER,
	YARDAGE INTEGER,
	HANDICAP INTEGER
)
/

CREATE TABLE ROUND(
 	ID INTEGER PRIMARY KEY,
	GOLFER_ID INTEGER REFERENCES GOLFER,
	TEE_ID INTEGER REFERENCES TEE,
	DIFFERENTIAL NUMBER,
  	DATE_PLAYED DATE
)
/

CREATE TABLE SCORE(
	ROUND_ID INTEGER REFERENCES ROUND,
	HOLE_ID REFERENCES HOLE,
	SCORE INTEGER
)
/

-- Trigger: trigger_differential on golf.rounds
CREATE OR REPLACE TRIGGER trigger_differential
AFTER INSERT OR UPDATE
ON ROUND
FOR EACH ROW
BEGIN
	set_differential(:NEW.ID);
END;
/

-- Trigger: trigger_handicap on golf.rounds
CREATE OR REPLACE TRIGGER trigger_handicap
AFTER INSERT OR UPDATE OR DELETE
ON round
FOR EACH ROW
BEGIN
	set_handicap(:NEW.GOLFER_ID);
END;
/
